import unittest
from ovos_utils import rotate_list, camel_case_split, camelize, \
    transliterate, titleize, parameterize, ordinalize, get_handler_name


class TestHelpers(unittest.TestCase):

    def test_inflection_utils(self):
        # just imports from inflection package
        # https://inflection.readthedocs.io/en/latest/
        self.assertEqual(camelize("device_type"), "DeviceType")
        self.assertEqual(titleize("MyAwesome_Skill"), "My Awesome Skill")
        self.assertEqual(transliterate('Ærøskøbing'), 'rskbing')
        self.assertEqual(parameterize("Donald E. Knuth"), 'donald-e-knuth')
        self.assertEqual(ordinalize(5), '5th')
        self.assertEqual(ordinalize(2), '2nd')

    def test_utils(self):
        def some_function():
            return

        self.assertEqual(get_handler_name(some_function), "some_function")
        self.assertEqual(get_handler_name(self.test_utils), "test_utils")

        self.assertEqual(camel_case_split("MyAwesomeSkill"),
                         "My Awesome Skill")

        self.assertEqual(rotate_list([1, 2, 3]), [2, 3, 1])
        self.assertEqual(rotate_list([1, 2, 3], 2), [3, 1, 2])
        self.assertEqual(rotate_list([1, 2, 3], 3), [1, 2, 3])

        self.assertEqual(rotate_list([1, 2, 3], -1), [3, 1, 2])
        self.assertEqual(rotate_list([1, 2, 3], -2), [2, 3, 1])
        self.assertEqual(rotate_list([1, 2, 3], -3), [1, 2, 3])



import unittest
from ovos_utils.parse import remove_parentheses, summarize, search_in_text, \
    extract_paragraphs, extract_sentences, split_sentences, singularize


class TestParseHelpers(unittest.TestCase):
    @classmethod
    def setUpClass(self):
        self.test_string = "Mr. Smith bought cheapsite.com for 1.5 million dollars, i.e. he paid a lot for it. Did he mind? Adam Jones Jr. thinks he didn't. In any case, this isn't true... Well, with a probability of .9 it isn't.I know right\nOK"
        self.wiki_dump = """Mycroft is a free and open-source voice assistant for Linux-based operating systems that uses a natural language user interface. Its code was formerly copyleft, but is now under a permissive license.

History
Inspiration for Mycroft came when Ryan Sipes and Joshua Montgomery were visiting the Kansas City makerspace, where they came across a simple and basic intelligent virtual assistant project. They were interested in the technology, but did not like its inflexibility. Montgomery believes that the burgeoning industry of intelligent personal assistance poses privacy concerns for users and has promised that Mycroft will protect privacy through its open source machine learning platform. Mycroft has won several awards including the prestigious Techweek's KC Launch competition in 2016. Mycroft was part of the Sprint Accelerator 2016 class in Kansas City and joined 500 Startups Batch 20 in February 2017. The company accepted a strategic investment from Jaguar Land Rover during this same time period. To date, the company has raised more than $2.5 million from institutional investors and has opted to offer shares of the company to the public through Startengine, an equity crowdfunding platform. It is named after a fictional computer from 1966 science fiction novel The Moon Is a Harsh Mistress.

Mycroft voice stack
Mycroft provides open source software for most parts of the voice stack.

Wake Word
Mycroft does Wake Word spotting, also called keyword spotting, through its Precise Wake Word engine. Prior to Precise becoming the default Wake Word engine, Mycroft employed PocketSphinx. Instead of being based on phoneme recognition, Precise uses a trained recurrent neural network to distinguish between sounds which are, and which aren't Wake Words.

Speech to text
Mycroft is partnering with Mozilla's Common Voice Project to leverage their DeepSpeech speech to text software.

Intent parsing
Mycroft uses an intent parser called Adapt to convert natural language into machine-readable data structures. Adapt undertakes intent parsing by matching specific keywords in an order within an utterance. They also have a parser, Padatious. Padatious, in contrast, uses example-based inference to determine intent.

Text to speech
For speech synthesis Mycroft uses Mimic, which is based on the Festival Lite speech synthesis system.

Modular design and interoperability
Mycroft is designed to be modular, so users are able to change its components. For example, espeak can be used instead of Mimic.

Hardware
The Mycroft project is also working on and selling smart speakers that run its software. All of its hardware is open-source, released under the CERN Open Hardware Licence.Its first hardware project was the Mark I, targeted primarily at developers. Its production was partially funded through a Kickstarter campaign, which finished successfully. Units started shipping out in April 2016.Its most recent hardware project is the Mark II, intended for general usage, not just for developers. Unlike the Mark I, the Mark II is equipped with a screen, being able to relay information both visually as well as acoustically. As with the Mark I, the Mark II's production will be partially funded through a Kickstarter campaign, which wrapped up in February 2018, hitting almost 8 times its original goal.Mycroft announced that a third hardware project, Mark III, will be offered through Kickstarter, and that an entire product line of Mark I, II, and III will be released to stores by November, 2019.

Partnerships
Mycroft has undertaken several commercial collaborations. In May 2018, the company partnered with WorkAround, an impact sourcing provider who broker work opportunities for refugees, to undertake bulk machine learning training. In October 2018, Mycroft collaborated with disease surveillance and forecasting company, SickWeather, to identify the frequency of coughing on public transport, funded by the City of Kansas City, Missouri.

See also

Amazon Alexa
Cortana
Google Assistant
Siri


== References ==
    """

    def test_singularize(self):
        # uses inflection module for english
        # https://pypi.org/project/inflection/
        self.assertEqual(
            singularize("dogs"), "dog"
        )
        self.assertEqual(
            singularize("dogs", "en"), "dog"
        )

        # failure cases, just strips "s"
        self.assertEqual(
            singularize("cães", "pt"), "cãe"
        )
        self.assertEqual(
            singularize("cães", "es"), "cãe"
        )

    # TODO fix me
    """
    def test_split_sentences(self):
        # no split
        self.assertEqual(split_sentences("A.E:I.O.U"), [])
        self.assertEqual(split_sentences("hello.com"), [])

        # ambiguous, nosplit
        self.assertEqual(split_sentences("hello. he said"), [])
        self.assertEqual(split_sentences("hello.he said"), [])
        # TODO maybe split this one?
        self.assertEqual(split_sentences("hello . he said"), [])

        # ambiguous, but will split
        self.assertEqual(
            split_sentences("hello.He said"),
            ["hello", "He said"]
        )

        # split at periods
        self.assertEqual(
            split_sentences("hello. He said"),
            ["hello", "He said"]
        )
        self.assertEqual(
            split_sentences("hello . He said"),
            ["hello", "He said"]
        )

    # TODO fix me
    def test_summarize(self):
        self.assertEqual(
            extract_paragraphs("precise", self.wiki_dump),
            []
        )
        self.assertEqual(
            extract_sentences("intent", self.wiki_dump),
            []
        )
        self.assertEqual(summarize(self.test_string), "")
    """



from ovos_utils.intents.intent_service_interface import IntentQueryApi, \
    IntentServiceInterface
from ovos_utils.intents.converse import ConverseTracker
from ovos_utils.intents.layers import IntentLayers



import requests


def detect_lang(text, return_dict=False, key=None,
                url="https://libretranslate.com/detect"):
    """host it yourself https://github.com/uav4geo/LibreTranslate"""
    params = {"q": text}
    if key:
        params["api_key"] = key
    res = requests.post(url, params=params).json()
    if return_dict:
        return res[0]
    return res[0]["language"]


if __name__ == "__main__":
    assert detect_lang("olá eu chamo-me joaquim") == "pt"

    assert detect_lang("olá eu chamo-me joaquim", return_dict=True) == \
           {'confidence': 0.9999939001351439, 'language': 'pt'}

    assert detect_lang("hello world") == "en"

    fr_en = """\
    France is the largest country in Western Europe and the third-largest in Europe as a whole.
    A accès aux chiens et aux frontaux qui lui ont été il peut consulter et modifier ses collections
    et exporter Cet article concerne le pays européen aujourd’hui appelé République française.
    Pour d’autres usages du nom France, Pour une aide rapide et effective, veuiller trouver votre aide
    dans le menu ci-dessus.
    Motoring events began soon after the construction of the first successful gasoline-fueled automobiles.
    The quick brown fox jumped over the lazy dog."""

    assert detect_lang(fr_en) == "fr"

    assert detect_lang("This piece of text is in English. Този текст е на Български.",
                              return_dict=True) == {'confidence': 0.28571342657428966, 'language': 'en'}



from setuptools import setup

setup(
    name='ovos_utils',
    version='0.0.14a1',
    packages=['ovos_utils',
              'ovos_utils.intents',
              'ovos_utils.sound',
              "ovos_utils.enclosure",
              'ovos_utils.enclosure.mark1',
              'ovos_utils.enclosure.mark1.eyes',
              'ovos_utils.enclosure.mark1.faceplate',
              'ovos_utils.skills',
              'ovos_utils.lang'],
    url='https://github.com/OpenVoiceOS/ovos_utils',
    install_requires=[
        "mycroft-messagebus-client",
        "pexpect",
        "pyxdg",
        "PyYAML",
        "kthread",
        "json_database",
        "requests",
        "colour",
        "inflection"],
    extras_require={
        "extras": ["phoneme_guesser", "colour", "rapidfuzz"]
    },
    include_package_data=True,
    license='Apache',
    author='jarbasAI',
    author_email='jarbasai@mailfence.com',
    description='collection of simple utilities for use across the mycroft ecosystem'
)



import subprocess
import time
from ovos_utils.log import LOG
from ovos_utils.signal import check_for_signal


def play_audio(uri, play_cmd="play %1"):
    """ Play a audio file.

        Returns: subprocess.Popen object
    """
    play_wav_cmd = str(play_cmd).split(" ")
    for index, cmd in enumerate(play_wav_cmd):
        if cmd == "%1":
            play_wav_cmd[index] = uri
    try:
        return subprocess.Popen(play_wav_cmd)
    except Exception as e:
        LOG.error("Failed to launch WAV: {}".format(play_wav_cmd))
        LOG.debug("Error: {}".format(repr(e)), exc_info=True)
        return None


def play_wav(uri, play_cmd="paplay %1"):
    """ Play a wav-file.

        Returns: subprocess.Popen object
    """
    play_wav_cmd = str(play_cmd).split(" ")
    for index, cmd in enumerate(play_wav_cmd):
        if cmd == "%1":
            play_wav_cmd[index] = uri
    try:
        return subprocess.Popen(play_wav_cmd)
    except Exception as e:
        LOG.error("Failed to launch WAV: {}".format(play_wav_cmd))
        LOG.debug("Error: {}".format(repr(e)), exc_info=True)
        return None


def play_mp3(uri, play_cmd="mpg123 %1"):
    """ Play a mp3-file.

        Returns: subprocess.Popen object
    """
    play_mp3_cmd = str(play_cmd).split(" ")
    for index, cmd in enumerate(play_mp3_cmd):
        if cmd == "%1":
            play_mp3_cmd[index] = uri
    try:
        return subprocess.Popen(play_mp3_cmd)
    except Exception as e:
        LOG.error("Failed to launch MP3: {}".format(play_mp3_cmd))
        LOG.debug("Error: {}".format(repr(e)), exc_info=True)
        return None


def play_ogg(uri, play_cmd="ogg123 -q %1"):
    """ Play a ogg-file.

        Returns: subprocess.Popen object
    """
    play_ogg_cmd = str(play_cmd).split(" ")
    for index, cmd in enumerate(play_ogg_cmd):
        if cmd == "%1":
            play_ogg_cmd[index] = uri
    try:
        return subprocess.Popen(play_ogg_cmd)
    except Exception as e:
        LOG.error("Failed to launch OGG: {}".format(play_ogg_cmd))
        LOG.debug("Error: {}".format(repr(e)), exc_info=True)
        return None


def record(file_path, duration, rate, channels):
    if duration > 0:
        return subprocess.Popen(
            ["arecord", "-r", str(rate), "-c", str(channels), "-d",
             str(duration), file_path])
    else:
        return subprocess.Popen(
            ["arecord", "-r", str(rate), "-c", str(channels), file_path])


def is_speaking():
    """Determine if Text to Speech is occurring

    Returns:
        bool: True while still speaking
    """
    return check_for_signal("isSpeaking", -1)


def wait_while_speaking():
    """Pause as long as Text to Speech is still happening

    Pause while Text to Speech is still happening.  This always pauses
    briefly to ensure that any preceeding request to speak has time to
    begin.
    """
    time.sleep(0.3)  # Wait briefly in for any queued speech to begin
    while is_speaking():
        time.sleep(0.1)


from ovos_utils.system import MycroftRootLocations
from ovos_utils.fingerprinting import detect_platform, MycroftPlatform
from enum import Enum
from os.path import exists


class MycroftEnclosures(str, Enum):
    PICROFT = "picroft"
    BIGSCREEN = "kde"
    OVOS = "OpenVoiceOS"
    OLD_MARK1 = "mycroft_mark_1(old)"
    MARK1 = "mycroft_mark_1"
    MARK2 = "mycroft_mark_2"
    HOLMESV = "HolmesV"
    OLD_HOLMES = "mycroft-lib"
    GENERIC = "generic"
    OTHER = "unknown"


def enclosure2rootdir(enclosure=None):
    enclosure = enclosure or detect_enclosure()
    if enclosure == MycroftEnclosures.OLD_MARK1:
        return MycroftRootLocations.OLD_MARK1
    elif enclosure == MycroftEnclosures.MARK1:
        return MycroftRootLocations.MARK1
    elif enclosure == MycroftEnclosures.MARK2:
        return MycroftRootLocations.MARK2
    elif enclosure == MycroftEnclosures.PICROFT:
        return MycroftPlatform.PICROFT
    elif enclosure == MycroftEnclosures.OVOS:
        return MycroftPlatform.OVOS
    elif enclosure == MycroftEnclosures.BIGSCREEN:
        return MycroftPlatform.BIGSCREEN
    return None


def detect_enclosure():
    platform = detect_platform()
    if platform == MycroftPlatform.MARK1:
        if exists(MycroftRootLocations.OLD_MARK1):
            return MycroftEnclosures.OLD_MARK1
        return MycroftEnclosures.MARK1
    elif platform == MycroftPlatform.MARK2:
        return MycroftEnclosures.MARK2
    elif platform == MycroftPlatform.PICROFT:
        return MycroftEnclosures.PICROFT
    elif platform == MycroftPlatform.OVOS:
        return MycroftEnclosures.OVOS
    elif platform == MycroftPlatform.BIGSCREEN:
        return MycroftEnclosures.BIGSCREEN
    elif platform == MycroftPlatform.HOLMESV:
        return MycroftEnclosures.HOLMESV
    elif platform == MycroftPlatform.OLD_HOLMES:
        return MycroftEnclosures.OLD_HOLMES

    return MycroftEnclosures.OTHER
